package statemachine.hsm;

/**
 * This class represents a Hierarchical State Machine (HSM) 
 * @author Alexei Krasnopolski (krasnop@bellsouth.net)
 */
abstract public class Hsm extends State 
{
  /**
   * Holds reference to current state of HSM 
   */
  private State currState;
  /**
   * Initializing of HSM. Moving from pseudostate (null) to initial state (this), 
   * root of HSM's state tree. 
   */  
  public void init()
  {
    enter();
    setState( this );  
  }
  /**
   * Get-method for current state 
   */  
  public State getState()
  {
    return currState;
  }
  /**
   * Set-method for current state. After setting currState field, we have to check
   * does have this state init transition? If it has HSM moves to up along branch of state tree. 
   */  
  public void setState( State dest )
  {
	currState = dest;
	State daughterNode;
	
	while( (daughterNode = currState.fireInit()) != null )
	{
	  (currState = daughterNode).enter();
	}
  }
  /**
   * Dispatch method for external events. Pass the event to the states. Current state
   * is always first. After processing event HSM checks return value and in case null 
   * the process is over. 
   */  
  public void dispatchEvent(final Event e )
  {
    State parentState = currState;
    Event ee = e;
    do {
//      System.out.println(">>> dispatch event " + e.getID() + "; state " + parentState );
      parentState = parentState.fireEvent( e );
    } while (parentState != null );
//    while( (parentState = parentState.fireEvent( e )) != null );
  }
}
