package statemachine.hsm;

/**
 * This class represents a graph of state behavioral inheritance
 * 
 * @author Alexei Krasnopolski (krasnop@bellsouth.net)
 */

class Path
{
  /**
   * reference to the root node of HSM state tree. It is common for all states of HSM
   */
  Hsm root;
  /**
   * Reference to the parent node for this node in tree hierarchy
   */
  State parent;
  /**
   * Reference to the daughter node for this node in tree hierarchy. This helper field trails
   * moving from leaf nodes to root. This allows make moving back on the same way.
   */
  State daughter;
  /**
   * Holds the index of a node's level in a tree of graph. Highest level equals 0 ( root node of
   * state machine )
   */
  int level;
  /**
   * Constructor creates instance of path for given instance of State (Path class is superclass of
   * State). It takes one parameter. The parameter is a reference to the parent node of the current
   * state in the state tree.
   */
  protected Path( State parent )
  {
    this.parent = parent;
    daughter = null;
    level = parent.level + 1;
    root = parent.root;
  }
  /**
   * Constructor without parameter uses for creates instance of root node.
   */
  protected Path()
  {
    parent = null;
    daughter = null;
    level = 0;
    root = (Hsm)this;
  }
  /**
   * Get-method returns reference to the parent node of the instance.
   */
  protected State getParent()
  {
    return parent;
  }
  /**
   * Method return parent node of state with <code>lvl</code> level from current path. Moving from
   * this node to <code>lvl</code>-level node is recording by setting <code>daughter</code>
   * fields for each processed node. If level of this node lows then <code>lvl</code> method
   * returns reference to the current instance.
   * 
   * @param lvl
   *          is level of node in the current path (branch) of SM tree
   * @return parent state of node lvl level
   */
  State pullDownToLevel( int lvl )
  {
    State tmp = (State)this;           // starting position of tmp is this state
    State prnt;
    while( tmp.level > lvl )          // if leval of tmp > lvl (the end poin of moving)
    {                                  // let's move
      if( (prnt = tmp.getParent()) == null)  // does tmp is root?
        return null;
      prnt.daughter = tmp;          // mark a way we are going: daughter will help us when we go opposit direction
      tmp = prnt;                    // next step
    }
    return tmp;
  }
  /**
   * Get-method returns reference to the root noode of the path which is common node for all
   * branches of state node tree of the HSM.
   */
  Hsm getRoot()
  {
    return root;
  };
  /**
   * Get-method returns reference to the level of the instance.
   */
  int getLevel()
  {
    return level;
  };
}
