package statemachine.hsm;

/**
 * This class represents a state of state machine
 * 
 * @author Alexei Krasnopolski (krasnop@bellsouth.net)
 */
abstract public class State extends Path
{
  /**
   * Constructor creates instance of State and invokes a constructor of superclass
   */
  public State( State parent )
  {
    super( parent );
  }

  /**
   * Constructor creates instance of root state (represents Hierarchical State Machine - HSM)
   */
  public State()
  {}

  /**
   * The method moves HSM from current state to destination state. The given instance is a state
   * stimulated this transition.
   */
  protected void transition( State destination )
  {
    int sourceLevel = this.getLevel(); // level of source node of the transition
// Moving along path of tree from current state to root of the tree.
// Note <code>this</code> is reference to state instance of transition source
// For each leaving state execute exit() method.
    State testedNode = root.getState();
// Adjust destination node: if it far away from root then current state, move it to current state level
    State destinationNode = destination.pullDownToLevel( testedNode.level );
    while( testedNode != null )
    {
//      if( testedNode.getLevel() <= sourceLevel )  // while do not reach of source node level skip a testing
//      {
        if( testedNode == destination && this == destination ) // self transition
        {  
          testedNode.exit();
          testedNode.enter();
          destinationNode = destination;  // needed?
          break;  
        }
        if( testedNode == destinationNode ) 
          break;  // reach a common node for a current state and a destination state branches
//      }
      testedNode.exit(); // execute exit procedure
      destinationNode = destinationNode.pullDownToLevel( testedNode.level - 1 );
      testedNode = testedNode.getParent();
    }
// Now move from common node to destination node in opposite direction.
// Other direction is a reason to execute enter() methods for each state in the path
    while( destinationNode != destination )
    {
      (destinationNode = destinationNode.daughter).enter();
    }
// Set SM in the destination state. Check and execute initial transition(s) for given state.
    root.setState( destination );
  }

// These abstract methods have to be implemented in subclasses. They are represents
// behavior of the state.
  abstract public State fireInit();

  abstract public void enter();

  abstract public State fireEvent( Event e );

  abstract public void exit();
}
