package test;


import java.io.*;

import statemachine.hsm.Event;
import statemachine.hsm.Hsm;
import statemachine.hsm.State;


/**
 * This class represents a test HSM implementation.
 * @author Alexei Krasnopolski (krasnop@bellsouth.net)
 */

public class Test extends Hsm
{
  int myFoo;

  public void init()
  {
    System.out.print("Top-INIT;" );
    myFoo = 0;
    super.init();
  };

  public State fireInit()
  {
    System.out.print("INIT>>s0;" );
    return s1;
  }

  public State fireEvent( Event e )
  {
    printMessage( e, "s0" );
    switch( e.getID() )
    {
      case 5 :
        transition( s211 );
        return null;
      default :
        break;
    }
    return getParent();
  };
  public void enter()
  {
    System.out.print("ENTRY>>s0;" );
  }
  public void exit()
  {
    System.out.print("EXIT<<s0;" );
  };

  Hsm s0 = this;                 // This is just alias of current instance
  State s1 = new State( s0 )
  {
    public State fireEvent( Event e )
    {
      printMessage( e, "s1" );
      switch( e.getID() )
      {
        case 1 :
          transition( s1 );
          return null;
        case 2 :
          transition( s11 );
          return null;
        case 3 :
          transition( s2 );
          return null;
        case 4 :
          transition( s0 );
          return null;
        case 6 :
          transition( s211 );
          return null;
        default :
          break;
      }
      return getParent();
    }
    public State fireInit()
    {
      System.out.print("INIT>>s1;" );
      return s11;
    }
    public void enter()
    {
      System.out.print("ENTRY>>s1;" );
    }
    public void exit()
    {
      System.out.print("EXIT<<s1;" );
    }
  };
  State s11 = new State( s1 )
  {
    public State fireEvent( Event e )
    {
      printMessage( e, "s11" );
      switch( e.getID() )
      {
        case 7 :
          transition( s211 );
          return null;
        case 8 :
          System.out.print("myFoo="+myFoo + ";");
          if ( myFoo == 1 )
          {
            myFoo = 0;
            return null;
          }
          break;
        default :
          break;
      }
      return getParent();
    }
    public State fireInit()
    {
      return null;
    }
    public void enter()
    {
      System.out.print("ENTRY>>s11;" );
    }
    public void exit()
    {
      System.out.print("EXIT<<s11;" );
    }
  };
  State s2 = new State( s0 )
  {
    public State fireEvent( Event e )
    {
      printMessage( e, "s2" );
      switch( e.getID() )
      {
        case 3 :
          transition( s1 );
          return null;
        case 6 :
          transition( s11 );
          return null;
        default :
          break;
      }
      return getParent();
    }
    public State fireInit()
    {
      System.out.print("INIT>>s2;" );
      return s21;
    }
    public void enter()
    {
      System.out.print("ENTRY>>s2;" );
    }
    public void exit()
    {
      System.out.print("EXIT<<s2;" );
    }
  };
  State s21 = new State( s2 )
  {
    public State fireEvent( Event e )
    {
      printMessage( e, "s21" );
      switch( e.getID() )
      {
        case 2 :
          transition( s211 );
          return null;
        case 8 :
          System.out.print("myFoo=" + myFoo + ";");
          if ( myFoo == 0 )
          {
            myFoo = 1;
            transition( s21 );
            return null;
          }
          break;
        default :
          break;
      }
      return getParent();
    }
    public State fireInit()
    {
      System.out.print("INIT>>s21;" );
      return s211;
    }
    public void enter()
    {
      System.out.print("ENTRY>>s21;" );
    }
    public void exit()
    {
      System.out.print("EXIT<<s21;" );
    }
  };
  State s211 = new State( s21 )
  {
    public State fireEvent( Event e )
    {
      printMessage( e, "s211" );
      switch( e.getID() )
      {
        case 4 :
          transition( s21 );
          return null;
        case 7 :
          transition( s0 );
          return null;
        default :
          break;
      }
      return getParent();
    }
    public State fireInit()
    {
      return null;
    }
    public void enter()
    {
      System.out.print("ENTRY>>s211;" );
    }
    public void exit()
    {
      System.out.print("EXIT<<s211;" );
    }
  };

  public static void printMessage( Event e, String stateName )
  {
    char c = 'A';
    System.out.print( "Event-" + (char)(c + e.getID() - 1) + ">>" + stateName + ";" );
  }
  
  public static void main(String[] args)
  {
    Test fsm = new Test();
    fsm.init();

    for(;;)
    {
      char c;
      try
      {
        System.out.print("\nSignal<-");
        c = (char)System.in.read();
        System.in.skip( 2 );
      }
      catch( IOException ioe )
      {
        System.out.print("\nIO error");
        break;
      }
      if( c >= 'a' && c <= 'h' )
      {
        fsm.dispatchEvent( new Event( fsm, c - 'a' + 1 ) );
      }
      else
        break;
    };
    System.out.print("\nThe Program Ends");
  }
}
